/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.dialogs;

import java.io.File;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import benten.ui.UiStatusException;
import benten.ui.fields.AbstractBrowseTextField;
import benten.ui.fields.IBrowseTextField;
import benten.ui.fields.IFolderTextField;
import benten.ui.fields.TextField;

/**
 * ワークフローで使用するダイアログの基底クラス。<br>
 * 翻訳ワークフローで必要な OK ボタン押下時の処理や検証が抽象化されています。
 * @author KASHIHARA Shinji
 */
public abstract class AbstractWorkflowDialog extends StatusDialog implements ModifyListener {

	/** デフォルト・ソース・ディレクトリー */
	protected static final String DEFAULT_SOURCE_DIR = ".benten/source";

	private final IStructuredSelection selection;
	private ModifyListener modifyListener;

	/**
	 * コンストラクター。
	 * @param parentShell 親シェル
	 * @param selection 選択
	 * @param title タイトル
	 */
	public AbstractWorkflowDialog(Shell parentShell, IStructuredSelection selection, String title) {
		super(parentShell);
		setTitle(title);
		setHelpAvailable(false);
		this.selection = selection;
	}

	@Override
	public void create() {
		super.create();
		updateStatus(getValidateStatus());
	}

	@Override
	final protected Control createDialogArea(Composite parent) {
		final Composite composite = (Composite) super.createDialogArea(parent);
		createControls(composite);
		return composite;
	}

	/**
	 * コントロールを作成するテンプレート・メソッド。
	 * @param composite コンポジット
	 */
	abstract public void createControls(Composite composite);

	/**
	 * 選択されたリソースを取得。
	 * @return 選択されたリソース
	 */
	protected IResource getSelectionResource() {
		final Object o = selection.getFirstElement();
		if (o != null && o instanceof IResource) {
			return (IResource) o;
		}
		return null;
	}

	/**
	 * 選択されたリソース文字列を取得。
	 * @return 選択されたリソース文字列
	 */
	protected String getSelectionResourceString() {
		final IResource resource = getSelectionResource();
		if (resource != null) {
			return resource.getLocation().toOSString();
		}
		return "";
	}

	@Override
	final protected boolean isResizable() {
		return true;
	}

	@Override
	final protected void okPressed() {
		if (hasError()) {
			return;
		}
		if (okPressedPrompt()) {
			final IStatus status = getResultStatus();
			updateStatus(status);
			super.okPressed();
		}
	}

	/**
	 * このダイアログに入力された値にエラーがあるか判定。
	 * @return エラーがある場合は true
	 */
	public boolean hasError() {
		modifyText(null);
		return getStatus().matches(IStatus.ERROR);
	}

	/**
	 * OK 押下時に確認ダイアログなどを表示する場合にオーバーライドするメソッド。
	 * @return 処理を続行する場合は true
	 */
	protected boolean okPressedPrompt() {
		return true;
	}

	/**
	 * 入力値ステータスの取得。
	 * @return 入力値ステータス
	 */
	abstract public IStatus getResultStatus();

	/**
	 * 変更リスナーをセット。
	 * @param modifyListener 変更リスナー
	 */
	public void setModifyListner(ModifyListener modifyListener) {
		this.modifyListener = modifyListener;
	}

	public void modifyText(ModifyEvent e) {
		if (modifyListener != null) {
			modifyListener.modifyText(e);
		}
		updateStatus(getValidateStatus());
	}

	/**
	 * 検証ステータスの取得。
	 * @return 検証ステータス
	 */
	public IStatus getValidateStatus() {
		try {
			return validate();
		} catch (final UiStatusException e) {
			return e.getStatus();
		}
	}

	/**
	 * 検証を行うテンプレート・メソッド。
	 * @return 検証ステータス
	 * @throws UiStatusException 検証エラーが発生した場合
	 */
	abstract public IStatus validate() throws UiStatusException;

	/**
	 * フィールドのパスを検証。
	 * @param field フィールド
	 * @throws UiStatusException 検証エラーが発生した場合
	 */
	protected void validatePath(IBrowseTextField field) throws UiStatusException {
		if (field == null) {
			return;
		}
		final String name = field.getLabelText();
		final String path = field.getText();
		if (path.length() == 0) {
			throw new UiStatusException(name + "を入力してください。");
		}
		final File file = field.getFile();
		if (file == null || !file.exists()) {
			throw new UiStatusException(name + "に指定されたリソースは存在しません。");
		}
		if (field instanceof IFolderTextField) {
			if (file.isFile()) {
				throw new UiStatusException(name + "にディレクトリーを指定してください。");
			}
		} else {
			if (file.isDirectory()) {
				throw new UiStatusException(name + "にファイルを指定してください。");
			}
			if (field instanceof AbstractBrowseTextField && ((AbstractBrowseTextField) field).invalidExtension()) {
				throw new UiStatusException(name + "の拡張子が不正です。");
			}
		}
	}

	/**
	 * フィールドの循環関係を検証。
	 * @param fromField 入力元フィールド
	 * @param toField 出力先フィールド
	 * @throws UiStatusException 検証エラーが発生した場合
	 */
	protected void validateFromTo(IBrowseTextField fromField, IBrowseTextField toField) throws UiStatusException {
		if (fromField == null || toField == null) {
			return;
		}
		String from = fromField.getFile().getAbsolutePath().replace('\\', '/') + "/";
		final String to = toField.getFile().getAbsolutePath().replace('\\', '/') + "/";
		if (from.equals(to)) {
			throw new UiStatusException(fromField.getLabelText() + "と" + toField.getLabelText() + "が同じです。");
		}
		if (toField instanceof IFolderTextField) {
			if (!(fromField instanceof IFolderTextField)) {
				from = fromField.getFile().getParentFile().getAbsolutePath().replace('\\', '/') + "/";
			}
			if (from.startsWith(to)) {
				throw new UiStatusException(toField.getLabelText() + "に" + fromField.getLabelText()
						+ "の上位ディレトクリーを指定することはできません。");
			}
			if (to.startsWith(from)) {
				throw new UiStatusException(toField.getLabelText() + "に" + fromField.getLabelText()
						+ "の下位ディレトクリーを指定することはできません。");
			}
		}
	}

	/**
	 * フィールドの必須チェック。
	 * @param field フィールド
	 * @throws UiStatusException 検証エラーが発生した場合
	 */
	protected void validateRequire(TextField field) throws UiStatusException {
		if (field == null) {
			return;
		}
		if (field.getText().equals("")) {
			throw new UiStatusException(field.getLabelText() + "を入力してください。");
		}
	}

	/**
	 * フィールドの正規表現チェック。
	 * @param field フィールド
	 * @param okPattern OK パターン
	 * @param ngPattern NG パターン
	 * @throws UiStatusException 検証エラーが発生した場合
	 */
	protected void validatePattern(TextField field, String okPattern, String ngPattern) throws UiStatusException {
		if (field == null) {
			return;
		}
		if (!field.getText().matches(okPattern)) {
			throw new UiStatusException(field.getLabelText() + "の形式が不正です。");
		}
		if (field.getText().matches(ngPattern)) {
			throw new UiStatusException(field.getLabelText() + "の形式が不正です。");
		}
	}
}
